VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "Sprite"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Ext_KEY = "SavedWithClassBuilder" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
Option Explicit

'This is a no fuss sprite class.  There is no consideration
'given to reuse of graphic data, ie 10 similar sprites will
'result in 10 copies of the graphic data and 10 copies of
'the mask data.

'This version is basic and does not cater for animation of
'The sprite.

'Useage:
'1. Declare a reference to a sprite
'   Public sprSomeName As Sprite
'2. Initialise the reference
'   Set sprSomeName = New Sprite
'3. Define the graphic data to be used, be sure to check return value is True!
'   b = sprSomeName.SetPicture(picTheGraphic, SrcX, SrcY, Width, Height)
'4. Position it
'   sprSomeName.MoveAbs x-coordinate, y-coordinate
'5. Activate it
'   sprSomeName.bActive = True
'
'6. In the program control loop do the following
'   a.  Restore backgrounds where sprite was last displayed
'       sprSomeName.Restore Form.hDC
'   b.  Do collision detection etc here
'   c.  Move the sprite
'       sprSomeName.MoveAbs newX,newY or sprSomeName.MoveRel dX, dY
'   d.  Draw the sprite
'       sprSomeName.Draw Form.hDC

'NOTE: To reduce flicker set the destination form (or picture box)
'AutoSave to true and then do a Form.Refresh _after_ drawing
'all sprites to the screen.  This has the effect of letting the
'system maintain an off-screen drawing area and saves having
'to create a separate DC.

'Class private variables
Private mbActive As Boolean 'True if sprite is to be displayed
Private mlGfxDC As Long     'DC for sprites graphic data
Private mlGfxBmp As Long    'Bitmap for sprites graphic data
Private mlMaskDC As Long    'DC for sprites Mask data
Private mlMaskBmp As Long   'Bitmap for sprites Mask data
Private mlSaveDC As Long    'DC for background saves
Private mlSaveBmp As Long   'Bitmap for background saves
Private mbSaved As Boolean  'True when bgrnd has been saved
Private mlW As Long         'Pixel width of the sprite
Private mlH As Long         'Pixel height of the sprite
Private mlX As Long         'X coord of sprite
Private mlY As Long         'Y coord of sprite

Public Property Let bActive(ByVal vData As Boolean)
Attribute bActive.VB_Description = "True if this sprite is to be displayed."
'Set when sprite has been initialised and is ready for use
'Syntax: X.bActive = True

'Only allow this toggle to be set if gfx data is loaded
If mlGfxDC Then mbActive = vData
End Property

Public Property Get bActive() As Boolean
'Syntax: Debug.Print X.iActive
    bActive = mbActive
End Property

Public Property Get lX() As Long
'Syntax: Debug.Print X.lX
    lX = mlX
End Property

Public Property Get lY() As Long
'Syntax: Debug.Print X.lY
    lY = mlY
End Property

Public Property Get lW() As Long
'Syntax: Debug.Print X.lW
    lW = mlW
End Property

Public Property Get lH() As Long
'Syntax: Debug.Print X.lH
    lH = mlH
End Property

Private Sub Class_Initialize()
'There is no specific initialisation code
End Sub

Private Sub Class_Terminate()
'Make sure all resources are released
FreeResources
End Sub

Public Sub Restore(rlDC As Long)
'Restore backgrounds where sprite was last drawn

Dim l As Long

'Only restore if something is saved
If mbSaved Then
    l = BitBlt(rlDC, mlX, mlY, mlW, mlH, mlSaveDC, 0, 0, SRCCOPY)
End If

'Clear flag
mbSaved = False

End Sub

Public Sub Draw(rlDC As Long)
'Draw the sprite at current position, making a save of the
'background first
Dim l As Long
Dim lColour As Long

'Only if sprite is valid
If mbActive Then
    'Copy data from background into sprites save DC where sprite is to be displayed
    l = BitBlt(mlSaveDC, 0, 0, mlW, mlH, rlDC, mlX, mlY, SRCCOPY)
    mbSaved = True

    'Copy sprite data into background using transparent blitting
    'This is a cludge, set back colour to white or transparency
    'does not work.  This needs investigating as things work
    'diffirent under VB 3 (16 bit code)!
    lColour = SetBkColor(rlDC, QBColor(15))
    l = BitBlt(rlDC, mlX, mlY, mlW, mlH, mlMaskDC, 0, 0, SRCAND)
    l = BitBlt(rlDC, mlX, mlY, mlW, mlH, mlGfxDC, 0, 0, SRCINVERT)
    lColour = SetBkColor(rlDC, lColour)
End If
End Sub

Public Sub Splat(rlDC As Long, rlX As Long, rlY As Long)
'This is a quick and dirty way of drawing the sprite into
'a DC without background saves etc.
Dim l As Long

'Only if sprite is valid
If mbActive Then
    'Copy sprite data into background using transparent blitting
    l = BitBlt(rlDC, rlX, rlY, mlW, mlH, mlMaskDC, 0, 0, SRCAND)
    l = BitBlt(rlDC, rlX, rlY, mlW, mlH, mlGfxDC, 0, 0, SRCINVERT)
End If

End Sub

Public Sub MoveAbs(rlX As Long, rlY As Long)
'Move to an absolute (x,y) position

mlX = rlX
mlY = rlY

End Sub

Public Sub MoveRel(rldX As Long, rldY As Long)
'Move relative to current position

mlX = mlX + rldX
mlY = mlY + rldY

End Sub

Public Function SetPicture(picGfx As PictureBox, rlX As Long, rlY As Long, rlW As Long, rlH As Long) As Boolean
'This initialises the sprite by allocating DC's and Bitmaps
'required for the sprites graphic, a mask for transparent
'BitBlting and a background save area.  The mask is also
'generated in the routine using method proposed in the VB
'Knowledge Base.

Dim l As Long
Dim lColour As Long
Dim lDC As Long
Dim lSrcDC As Long
Dim lBmp As Long
Dim bRetVal As Boolean

'Can only be called once
If mlGfxDC Then Exit Function

'Get working DC
lSrcDC = picGfx.hdc

'Create a DC and bitmap to hold the sprite graphics
mlGfxDC = CreateCompatibleDC(lSrcDC)
If mlGfxDC Then
    mlGfxBmp = CreateCompatibleBitmap(lSrcDC, rlW, rlH)

    'Copy graphics into DC
    If mlGfxBmp Then
        mlGfxBmp = SelectObject(mlGfxDC, mlGfxBmp)
        l = BitBlt(mlGfxDC, 0, 0, rlW, rlH, lSrcDC, rlX, rlY, SRCCOPY)

        'Create a DC and bitmap to hold the sprite mask
        mlMaskDC = CreateCompatibleDC(lSrcDC)
        If mlMaskDC Then
            mlMaskBmp = CreateBitmap(rlW, rlH, 1, 1, ByVal 0&)

            'Create masks
            If mlMaskBmp Then
                mlMaskBmp = SelectObject(mlMaskDC, mlMaskBmp)
                lColour = SetBkColor(mlGfxDC, QBColor(0))
                l = BitBlt(mlMaskDC, 0, 0, rlW, rlH, mlGfxDC, 0, 0, SRCCOPY)
                lColour = SetBkColor(mlGfxDC, lColour)

                'Create a  DC and bitmap for background saves
                mlSaveDC = CreateCompatibleDC(lSrcDC)
                If mlSaveDC Then
                    mlSaveBmp = CreateCompatibleBitmap(lSrcDC, rlW, rlH)
                    'Flag success and store variables
                    If mlSaveBmp Then
                        mlSaveBmp = SelectObject(mlSaveDC, mlSaveBmp)
                        bRetVal = True
                        mlW = rlW
                        mlH = rlH
                    End If
                End If
            End If
        End If
    End If
End If

'On failure free all resources
If Not bRetVal Then FreeResources

'Return success
SetPicture = bRetVal
End Function

Private Sub FreeResources()
'Free up all DC's and Bitmaps used by this sprite

Dim l As Long

'Free sprite gfx bitmap
If mlGfxBmp Then
    mlGfxBmp = SelectObject(mlGfxDC, mlGfxBmp)
    l = DeleteObject(mlGfxBmp)
End If

'Free sprite gfx DC
If mlGfxDC Then l = DeleteDC(mlGfxDC)

'Free mask bitmap
If mlMaskBmp Then
    mlMaskBmp = SelectObject(mlMaskDC, mlMaskBmp)
    l = DeleteObject(mlMaskBmp)
End If

'Free mask DC
If mlMaskDC Then l = DeleteDC(mlMaskDC)

'Free save bitmap
If mlSaveBmp Then
    mlSaveBmp = SelectObject(mlSaveDC, mlSaveBmp)
    l = DeleteObject(mlSaveBmp)
End If

'Free save DC
If mlSaveDC Then l = DeleteDC(mlSaveDC)

'Null all variables
mlGfxDC = 0
mlGfxBmp = 0
mlMaskDC = 0
mlMaskBmp = 0
mlSaveDC = 0
mlSaveBmp = 0

End Sub

Public Function lXMax() As Long
'Return maximum x position occupied by sprite
lXMax = mlX + mlW - 1
End Function

Public Function lYMax() As Long
'Return maximum y position occupied by sprite
lYMax = mlY + mlH - 1
End Function

Public Function bCollision(sprEnemy As Sprite) As Boolean
'See if the bounding rectangle of this sprite overlaps with
'that of the specified sprite.

Dim bRetVal As Boolean

'Both sprites must be active
If Not (mbActive And sprEnemy.bActive) Then Exit Function

'Default to true
bRetVal = True

'Check if collision is impossible
'Check if to left
If ((sprEnemy.lX + mlW - 1) < mlX) Then bRetVal = False
'Check if above
If ((sprEnemy.lY + sprEnemy.lH - 1) < mlY) Then bRetVal = False
'Check if to right
If (sprEnemy.lX > (mlX + mlW - 1)) Then bRetVal = False
'Check if below
If (sprEnemy.lY > (mlY + mlH - 1)) Then bRetVal = False

'Return success
bCollision = bRetVal

End Function
